PetBattle = select(2, ...)
PetBattle.modules, PetBattle.base_modules = {}, {}
PetBattle.MAX_PETS = NUM_BATTLE_PETS_IN_BATTLE or 3
PetBattle.MAX_ABILITIES = NUM_BATTLE_PET_ABILITIES or 3

local L = PetBattle.L

local battleChangedEvents = {"PET_BATTLE_PET_CHANGED", "PET_BATTLE_TURN_STARTED", "PET_BATTLE_PET_ROUND_PLAYBACK_COMPLETE"}
function PetBattle:OnInitialize()
	if( not PetBattleTacticianDB ) then PetBattleTacticianDB = {showVs = true, showAbilities = true} end
	self.defaults = {
		profile = {
			locked = false,
			enemyVs = {


			},
			friendlyAbilities = {
				enabled = true,
				active = false,
				inactive = true,
				scale = 1,
				alpha = 1
			},
			enemyAbilities = {
				enabled = true,
				active = true,
				inactive = true,
				scale = 1,
				alpha = 1
			}
		},
	}
			
	-- Initialize DB
	self.db = LibStub:GetLibrary("AceDB-3.0"):New("PBTacticianDB", self.defaults, true)
	self.db.RegisterCallback(self, "OnProfileChanged", "Reload")
	self.db.RegisterCallback(self, "OnProfileReset", "Reload")

	-- Alias the events that require us to update
	for _, event in pairs(battleChangedEvents) do
		self:RegisterEvent(event)
		self[event] = self.Update
	end

	-- Initial ones we always care about
	self:RegisterEvent("PET_BATTLE_OPENING_START")
	self:RegisterEvent("PET_BATTLE_CLOSE")

	-- And in case we reloaded 
	if( C_PetBattles.IsInBattle() ) then
		self:PET_BATTLE_OPENING_START()
	end
end

function PetBattle:PET_BATTLE_OPENING_START()
	self.battleSetup = true
	self:TriggerModules("CreateUI")
	self:Update()
end

function PetBattle:PET_BATTLE_CLOSE()
	self:TriggerModules("Reset")
	self.battleSetup = nil
end

function PetBattle:Update()
	-- PET_BATTLE_PET_CHANGED fires before PET_BATTLE_OPENING_DONE
	if( not self.battleSetup ) then return end

	local battlePets = {[LE_BATTLE_PET_ALLY] = {}, [LE_BATTLE_PET_ENEMY] = {}}

	for ownerType, pets in pairs(battlePets) do
		local activeSlot = C_PetBattles.GetActivePet(ownerType)

		for petSlot=1, C_PetBattles.GetNumPets(ownerType) do
			local speciesID = C_PetBattles.GetPetSpeciesID(ownerType, petSlot)
			if( speciesID ) then
				local name, icon, petType, creatureID, sourceText, description, isWild, canBattle, tradable, unique = C_PetJournal.GetPetInfoBySpeciesID(speciesID)

				local pet = {["name"] = name, ["icon"] = icon, ["petType"] = petType, ["speciesID"] = speciesID}
				pet["icon"] = C_PetBattles.GetIcon(ownerType, petSlot)
				pet["active"] = activeSlot == petSlot

				-- Figure out the stats
				pet["quality"] = C_PetBattles.GetBreedQuality(ownerType, petSlot)
				pet["level"] = C_PetBattles.GetLevel(ownerType, petSlot)
				pet["speed"] = C_PetBattles.GetSpeed(ownerType, petSlot)
				pet["power"] = C_PetBattles.GetPower(ownerType, petSlot)
				pet["health"] = C_PetBattles.GetHealth(ownerType, petSlot)
				pet["maxhealth"] = C_PetBattles.GetMaxHealth(ownerType, petSlot)

				-- Load abilities
				pet["abilities"] = {}
				for abilitySlot=1, NUM_BATTLE_PET_ABILITIES do
					local id, name, icon, maxCooldown, description, numTurns, abilityPetType, noStrongWeakHints = C_PetBattles.GetAbilityInfo(ownerType, petSlot, abilitySlot);
					if( id ) then
						local ability = {["id"] = id, ["name"] = name, ["icon"] = icon, ["petType"] = abilityPetType, ["hints"] = not noStrongWeakHints}
						ability["usable"], ability["cooldown"] = C_PetBattles.GetAbilityState(ownerType, petSlot, abilitySlot)

						pet["abilities"][abilitySlot] = ability
					end
				end

				pets[petSlot] = pet
			end
		end
	end

	self:TriggerModules("Update", battlePets)
end

-- Tooltips
function PetBattle.ShowPetTooltip(self, anchor)
	PetBattleUnitTooltip_Attach(PetBattlePrimaryUnitTooltip, "TOPLEFT", anchor or self, "BOTTOMLEFT", -4, -10)
	PetBattleUnitTooltip_UpdateForUnit(PetBattlePrimaryUnitTooltip, self.petOwner, self.petSlot)
	PetBattlePrimaryUnitTooltip:Show()
end

function PetBattle.HidePetTooltip(self)
	PetBattlePrimaryUnitTooltip:Hide()
end

function PetBattle.ShowAbilityTooltip(self)
	PetBattleAbilityTooltip_SetAbility(self.petOwner, self.petSlot, self.petAbilitySlot)
	PetBattleAbilityTooltip_Show("TOPLEFT", self, "TOPLEFT", -4, -24)
end

function PetBattle.HideAbilityTooltip(self)
	PetBattlePrimaryAbilityTooltip:Hide()
end

function PetBattle.HideTooltip(self)
	GameTooltip:Hide()
end

-- Module management
function PetBattle:RegisterModule(type, parent)
	if( parent ) then
		self.modules[type] = setmetatable({}, {__index = self.base_modules[parent]})
	else
		self.modules[type] = {}
	end

	return self.modules[type]
end

function PetBattle:RegisterBaseModule(type)
	self.base_modules[type] = {}
	return self.base_modules[type]
end

function PetBattle:TriggerModules(type, ...)
	for name, obj in pairs(self.modules) do
		if( obj[type] ) then
			obj[type](obj, ...)
		end
	end
end

-- Options
function PetBattle:Reload()
	self:TriggerModules("SetStatus")
	self:TriggerModules("UpdateUI")
	self:Update()
end

-- Slash commands
function PetBattle:Print(msg)
	DEFAULT_CHAT_FRAME:AddMessage("|cff33ff99Pet Battle Tactician|r: " .. msg)
end

-- Event dispatch
local frame = CreateFrame("Frame")
frame:RegisterEvent("ADDON_LOADED")
frame:SetScript("OnEvent", function(self, event, addon)
	if( addon == "PetBattleTactician" ) then
		frame:UnregisterEvent("ADDON_LOADED")
		frame:SetScript("OnEvent", function(self, event, ...)
			PetBattle[event](PetBattle, event, ...)
		end)

		PetBattle:OnInitialize()
	end
end)

function PetBattle:RegisterEvent(event) frame:RegisterEvent(event) end
function PetBattle:UnregisterEvent(event) frame:UnregisterEvent(event) end
